@@ -1,18 +1,13 @@
-from __future__ import absolute_import, unicode_literals
-
 import re
-from collections import OrderedDict
 
 from django import template
 from django.template import loader
-from django.utils import six
-from django.utils.encoding import force_text, iri_to_uri
+from django.urls import NoReverseMatch, reverse
+from django.utils.encoding import iri_to_uri
 from django.utils.html import escape, format_html, smart_urlquote
-from django.utils.safestring import SafeData, mark_safe
+from django.utils.safestring import mark_safe
 
-from rest_framework.compat import (
-    NoReverseMatch, apply_markdown, pygments_highlight, reverse
-)
+from rest_framework.compat import apply_markdown, pygments_highlight
 from rest_framework.renderers import HTMLFormRenderer
 from rest_framework.utils.urls import replace_query_param
 
@@ -53,10 +48,10 @@ def with_location(fields, location):
 @register.simple_tag
 def form_for_link(link):
     import coreschema
-    properties = OrderedDict([
-        (field.name, field.schema or coreschema.String())
+    properties = {
+        field.name: field.schema or coreschema.String()
         for field in link.fields
-    ])
+    }
     required = [
         field.name
         for field in link.fields
@@ -124,7 +119,7 @@ def optional_docs_login(request):
 
 
 @register.simple_tag
-def optional_logout(request, user):
+def optional_logout(request, user, csrf_token):
     """
     Include a logout snippet if REST framework's logout view is in the URLconf.
     """
@@ -140,11 +135,16 @@ def optional_logout(request, user):
             <b class="caret"></b>
         </a>
         <ul class="dropdown-menu">
-            <li><a href='{href}?next={next}'>Log out</a></li>
+            <form id="logoutForm" method="post" action="{href}?next={next}">
+                <input type="hidden" name="csrfmiddlewaretoken" value="{csrf_token}">
+            </form>
+            <li>
+                <a href="#" onclick='document.getElementById("logoutForm").submit()'>Log out</a>
+            </li>
         </ul>
     </li>"""
-    snippet = format_html(snippet, user=escape(user), href=logout_url, next=escape(request.path))
-
+    snippet = format_html(snippet, user=escape(user), href=logout_url,
+                          next=escape(request.path), csrf_token=csrf_token)
     return mark_safe(snippet)
 
 
@@ -176,7 +176,7 @@ def as_list_of_strings(value):
 @register.filter
 def add_class(value, css_class):
     """
-    http://stackoverflow.com/questions/4124220/django-adding-css-classes-when-rendering-form-fields-in-a-template
+    https://stackoverflow.com/questions/4124220/django-adding-css-classes-when-rendering-form-fields-in-a-template
 
     Inserts classes into template variables that contain HTML tags,
     useful for modifying forms without needing to change the Form objects.
@@ -188,7 +188,7 @@ def add_class(value, css_class):
     In the case of REST Framework, the filter is used to add Bootstrap-specific
     classes to the forms.
     """
-    html = six.text_type(value)
+    html = str(value)
     match = class_re.search(html)
     if match:
         m = re.search(r'^%s$|^%s\s|\s%s\s|\s%s$' % (css_class, css_class,
@@ -205,12 +205,12 @@ def add_class(value, css_class):
 @register.filter
 def format_value(value):
     if getattr(value, 'is_hyperlink', False):
-        name = six.text_type(value.obj)
+        name = str(value.obj)
         return mark_safe('<a href=%s>%s</a>' % (value, escape(name)))
     if value is None or isinstance(value, bool):
         return mark_safe('<code>%s</code>' % {True: 'true', False: 'false', None: 'null'}[value])
     elif isinstance(value, list):
-        if any([isinstance(item, (list, dict)) for item in value]):
+        if any(isinstance(item, (list, dict)) for item in value):
             template = loader.get_template('rest_framework/admin/list_value.html')
         else:
             template = loader.get_template('rest_framework/admin/simple_list_value.html')
@@ -220,9 +220,9 @@ def format_value(value):
         template = loader.get_template('rest_framework/admin/dict_value.html')
         context = {'value': value}
         return template.render(context)
-    elif isinstance(value, six.string_types):
+    elif isinstance(value, str):
         if (
-            (value.startswith('http:') or value.startswith('https:')) and not
+            (value.startswith('http:') or value.startswith('https:') or value.startswith('/')) and not
             re.search(r'\s', value)
         ):
             return mark_safe('<a href="{value}">{value}</a>'.format(value=escape(value)))
@@ -230,17 +230,21 @@ def format_value(value):
             return mark_safe('<a href="mailto:{value}">{value}</a>'.format(value=escape(value)))
         elif '\n' in value:
             return mark_safe('<pre>%s</pre>' % escape(value))
-    return six.text_type(value)
+    return str(value)
 
 
 @register.filter
 def items(value):
     """
     Simple filter to return the items of the dict. Useful when the dict may
-    have a key 'items' which is resolved first in Django tempalte dot-notation
+    have a key 'items' which is resolved first in Django template dot-notation
     lookup.  See issue #4931
     Also see: https://stackoverflow.com/questions/15416662/django-template-loop-over-dictionary-items-with-items-as-key
     """
+    if value is None:
+        # `{% for k, v in value.items %}` doesn't raise when value is None or
+        # not in the context, so neither should `{% for k, v in value|items %}`
+        return []
     return value.items()
 
 
@@ -272,7 +276,7 @@ def schema_links(section, sec_key=None):
             links.update(new_links)
 
     if sec_key is not None:
-        new_links = OrderedDict()
+        new_links = {}
         for link_key, link in links.items():
             new_key = NESTED_FORMAT % (sec_key, link_key)
             new_links.update({new_key: link})
@@ -285,7 +289,7 @@ def schema_links(section, sec_key=None):
 def add_nested_class(value):
     if isinstance(value, dict):
         return 'class=nested'
-    if isinstance(value, list) and any([isinstance(item, (list, dict)) for item in value]):
+    if isinstance(value, list) and any(isinstance(item, (list, dict)) for item in value):
         return 'class=nested'
     return ''
 
@@ -311,85 +315,6 @@ def smart_urlquote_wrapper(matched_url):
         return None
 
 
-@register.filter
-def urlize_quoted_links(text, trim_url_limit=None, nofollow=True, autoescape=True):
-    """
-    Converts any URLs in text into clickable links.
-
-    Works on http://, https://, www. links, and also on links ending in one of
-    the original seven gTLDs (.com, .edu, .gov, .int, .mil, .net, and .org).
-    Links can have trailing punctuation (periods, commas, close-parens) and
-    leading punctuation (opening parens) and it'll still do the right thing.
-
-    If trim_url_limit is not None, the URLs in link text longer than this limit
-    will truncated to trim_url_limit-3 characters and appended with an ellipsis.
-
-    If nofollow is True, the URLs in link text will get a rel="nofollow"
-    attribute.
-
-    If autoescape is True, the link text and URLs will get autoescaped.
-    """
-    def trim_url(x, limit=trim_url_limit):
-        return limit is not None and (len(x) > limit and ('%s...' % x[:max(0, limit - 3)])) or x
-
-    safe_input = isinstance(text, SafeData)
-    words = word_split_re.split(force_text(text))
-    for i, word in enumerate(words):
-        if '.' in word or '@' in word or ':' in word:
-            # Deal with punctuation.
-            lead, middle, trail = '', word, ''
-            for punctuation in TRAILING_PUNCTUATION:
-                if middle.endswith(punctuation):
-                    middle = middle[:-len(punctuation)]
-                    trail = punctuation + trail
-            for opening, closing in WRAPPING_PUNCTUATION:
-                if middle.startswith(opening):
-                    middle = middle[len(opening):]
-                    lead = lead + opening
-                # Keep parentheses at the end only if they're balanced.
-                if (
-                    middle.endswith(closing) and
-                    middle.count(closing) == middle.count(opening) + 1
-                ):
-                    middle = middle[:-len(closing)]
-                    trail = closing + trail
-
-            # Make URL we want to point to.
-            url = None
-            nofollow_attr = ' rel="nofollow"' if nofollow else ''
-            if simple_url_re.match(middle):
-                url = smart_urlquote_wrapper(middle)
-            elif simple_url_2_re.match(middle):
-                url = smart_urlquote_wrapper('http://%s' % middle)
-            elif ':' not in middle and simple_email_re.match(middle):
-                local, domain = middle.rsplit('@', 1)
-                try:
-                    domain = domain.encode('idna').decode('ascii')
-                except UnicodeError:
-                    continue
-                url = 'mailto:%s@%s' % (local, domain)
-                nofollow_attr = ''
-
-            # Make link.
-            if url:
-                trimmed = trim_url(middle)
-                if autoescape and not safe_input:
-                    lead, trail = escape(lead), escape(trail)
-                    url, trimmed = escape(url), escape(trimmed)
-                middle = '<a href="%s"%s>%s</a>' % (url, nofollow_attr, trimmed)
-                words[i] = mark_safe('%s%s%s' % (lead, middle, trail))
-            else:
-                if safe_input:
-                    words[i] = mark_safe(word)
-                elif autoescape:
-                    words[i] = escape(word)
-        elif safe_input:
-            words[i] = mark_safe(word)
-        elif autoescape:
-            words[i] = escape(word)
-    return ''.join(words)
-
-
 @register.filter
 def break_long_headers(header):
     """
@@ -397,5 +322,5 @@ def break_long_headers(header):
     when possible (are comma separated)
     """
     if len(header) > 160 and ',' in header:
-        header = mark_safe('<br> ' + ', <br>'.join(header.split(',')))
+        header = mark_safe('<br> ' + ', <br>'.join(escape(header).split(',')))
     return header
